@@ -4,6 +4,7 @@ import os
 from importlib import import_module
 
 from pagador import settings
+from pagador.utils import Formatador
 
 
 class TemplateNaoEncontrado(Exception):
@@ -42,6 +43,7 @@ class TipoDeCampo(object):
 
 class FormatoDeCampo(object):
     texto = "texto"
+    ascii = "ascii"
     html = "html"
     xml = "xml"
     json = "json"
@@ -136,6 +138,8 @@ class FormularioBase(object):
                 valor = valores.get(field.nome, None)
                 if valor and field.formato == FormatoDeCampo.json:
                     valor = json.loads(valor)
+                if valor and field.formato == FormatoDeCampo.ascii:
+                    valor = Formatador.string_para_ascii(valor)
                 setattr(model, field.nome, valor)
         return model
 
